@@ -28,26 +28,37 @@
 #include "Spell.h"
 #include "dictionary.h"
 #include "checker.h"
+#include "control.h"
 
 CORBA_Environment ev;
 CORBA_ORB orb;
 
 static BonoboGenericFactory *dictionary_factory;
 static BonoboGenericFactory    *checker_factory;
+static BonoboGenericFactory    *control_factory;
 
 static int active_dictionary_servers;
 static int active_checker_servers;
+static int active_control_servers;
+
+typedef enum {
+	TYPE_DICTIONARY,
+	TYPE_CHECKER,
+	TYPE_CONTROL,
+} SpellObjectType;
 
 static void
 init_server_factory (int argc, char **argv)
 {
-        gnome_init_with_popt_table("spell-factory", VERSION,
+        gnome_init_with_popt_table("gnome-spell-component", VERSION,
 				   argc, argv,
 				   oaf_popt_options, 0, NULL); 
 	orb = oaf_init (argc, argv);
 
 	if (bonobo_init (orb, NULL, NULL) == FALSE)
 		g_error (_("I could not initialize Bonobo"));
+
+	glade_gnome_init ();
 }
 
 static void
@@ -57,77 +68,71 @@ try_quit ()
 		return;
 	bonobo_object_unref (BONOBO_OBJECT (checker_factory));
 	bonobo_object_unref (BONOBO_OBJECT (dictionary_factory));
+
 	gtk_main_quit ();
 }
 
 static void
-dictionary_destroyed (GtkObject *checker_object)
+object_destroyed (GtkObject *checker_object, gpointer data)
 {
-	active_dictionary_servers--;
-
-	try_quit ();
-}
-
-static void
-checker_destroyed (GtkObject *checker_object)
-{
-	active_checker_servers--;
+	(*(gint *) data) --;
 
 	try_quit ();
 }
 
 static BonoboObject *
-factory_checker_new (BonoboGenericFactory *this_factory, void *data)
+factory_object_new (BonoboGenericFactory *this_factory, gpointer data)
 {
-	GNOMESpellChecker *checker;
+	GtkObject *new_object;
+	gint *counter;
 
-	checker = gnome_spell_checker_new ();
-	
-	if (!checker)
+	switch (GPOINTER_TO_INT (data)) {
+	case TYPE_DICTIONARY:
+		new_object = GTK_OBJECT (gnome_spell_dictionary_new ());
+		counter    = &active_dictionary_servers;
+		break;
+	case TYPE_CHECKER:
+		new_object = GTK_OBJECT (gnome_spell_checker_new ());
+		counter    = &active_checker_servers;
+		break;
+	case TYPE_CONTROL:
+		new_object = GTK_OBJECT (gnome_spell_control_new ());
+		counter    = &active_control_servers;
+		break;
+	default:
+		g_warning ("Dunno how to create object of this unknown type");
+		return NULL;
+	}
+	if (!new_object)
 		return NULL;
 
-	active_checker_servers++;
+	(*counter) ++;
 
 	gtk_signal_connect (
-		GTK_OBJECT (checker), "destroy",
-		checker_destroyed, NULL);
+		new_object, "destroy", object_destroyed, counter);
 
-	return BONOBO_OBJECT (checker);
-}
-
-static BonoboObject *
-factory_dictionary_new (BonoboGenericFactory *this_factory, void *data)
-{
-	GNOMESpellDictionary *dictionary;
-
-	dictionary = gnome_spell_dictionary_new ();
-	
-	if (!dictionary)
-		return NULL;
-
-	active_dictionary_servers++;
-
-	gtk_signal_connect (
-		GTK_OBJECT (dictionary), "destroy",
-		dictionary_destroyed, NULL);
-
-	return BONOBO_OBJECT (dictionary);
+	return BONOBO_OBJECT (new_object);
 }
 
 static void
 factory_init (void)
 {
-	checker_factory = bonobo_generic_factory_new (
-		"OAFIID:GNOME_Spell_CheckerFactory:0.1",
-		factory_checker_new, NULL);
-	dictionary_factory = bonobo_generic_factory_new (
-		"OAFIID:GNOME_Spell_DictionaryFactory:0.1",
-		factory_dictionary_new, NULL);
+	checker_factory = bonobo_generic_factory_new
+		("OAFIID:GNOME_Spell_CheckerFactory:" API_VERSION,
+		 factory_object_new, GINT_TO_POINTER (TYPE_CHECKER));
+	dictionary_factory = bonobo_generic_factory_new
+		("OAFIID:GNOME_Spell_DictionaryFactory:" API_VERSION,
+		 factory_object_new, GINT_TO_POINTER (TYPE_DICTIONARY));
+	control_factory = bonobo_generic_factory_new
+		("OAFIID:GNOME_Spell_ControlFactory:" API_VERSION,
+		 factory_object_new, GINT_TO_POINTER (TYPE_CONTROL));
 
 	if (!checker_factory)
 		g_error ("It was not possible to register a new checker factory");
 	if (!dictionary_factory)
 		g_error ("It was not possible to register a new dictionary factory");
+	if (!control_factory)
+		g_error ("It was not possible to register a new spell control factory");
 }
 
 int
